@@ -1,5 +1,3 @@
-from infrahub import lock
-from infrahub.core.registry import registry
 from infrahub.message_bus import messages
 from infrahub.services import InfrahubServices
 from infrahub.tasks.registry import refresh_branches
@@ -24,8 +22,7 @@ async def rebased_branch(message: messages.RefreshRegistryRebasedBranch, service
         )
         return
 
-    async with lock.registry.local_schema_lock():
-        service.log.info("Refreshing rebased branch")
+    async with service.database.start_session(read_only=True) as db:
+        await refresh_branches(db=db)
 
-        async with service.database.start_session(read_only=True) as db:
-            registry.branch[message.branch] = await registry.branch_object.get_by_name(name=message.branch, db=db)
+    await service.component.refresh_schema_hash()
